import UIKit
import PlaygroundSupport

/**
Oberfläche zur Verwaltung der Warteschlange

- version: 1.0
- author: Klaus Reinold
*/
class Anzeige: UIView
{
    /** Überschrift */
    private let labelÜberschrift: UILabel
    
    /** Unterüberschrift */
    private let labelUnterüberschrift: UILabel
    
    /** Schaltfläche zum Aufruf der Methode zum Einfügen eines Patienten, der im Textfeld eingetragen wurde */
    private let knopfEinfügen: UIButton
    
    /** Textfeld zum Eintrag eines neuen Patienten */
    private let textfeldEinfügen: UITextField
    
    /** Schaltfläche zum Aufruf der Methode zum Entfernen und Anzeigen eines Patienten */
    private let knopfEntfernen: UIButton
    
    /** Beschriftung */
    private let labelAufnehmen: UILabel
    
    /** Beschriftung */
    private let labelEntfernen: UILabel
    
    /** Label zur Anzeige des nächsten Patienten */
    private let labelNächster: UILabel
    
    /** Verwaltete Warteschlange, an die Botschaften über Ereignisse weitergeleitet werden */
    var w: Warteschlange
    
    /**
     Konstruktor der Oberfläche
     Erzeugt ein Fenster mit den zugehörigen Komponenten
     */
    override init(frame: CGRect)
    {
        // Frame-Initialisierung
        labelÜberschrift = UILabel()
        labelUnterüberschrift = UILabel()
        knopfEinfügen = UIButton(type: .roundedRect)
        textfeldEinfügen = UITextField()
        knopfEntfernen = UIButton(type: .roundedRect)
        labelAufnehmen = UILabel()
        labelEntfernen = UILabel()
        labelNächster = UILabel()
        w = Warteschlange()
        super.init(frame: frame)
        
        // Anfang Komponenten
        labelÜberschrift.frame = CGRect(x: 10, y: 10, width: frame.width - 16, height: 20)
        labelÜberschrift.text = "Warteschlangenverwaltung"
        labelÜberschrift.textAlignment = .center
        addSubview(labelÜberschrift)
        labelUnterüberschrift.frame = CGRect(x: 10, y: 30, width: frame.width - 16, height: 20)
        labelUnterüberschrift.text = "Praxis Dr. Ted Tanus"
        labelUnterüberschrift.textAlignment = .center
        addSubview(labelUnterüberschrift)
        labelAufnehmen.frame = CGRect(x: 10, y: 65, width: 380, height: 20)
        labelAufnehmen.text = "Person in Warteschlange aufnehmen"
        addSubview(labelAufnehmen)
        textfeldEinfügen.frame = CGRect(x: 10, y: 90, width: 270, height: 20)
        textfeldEinfügen.placeholder = "Name eingeben"
        addSubview(textfeldEinfügen)
        knopfEinfügen.frame = CGRect(x: 280, y: 85, width: 80, height: 30)
        knopfEinfügen.setTitle("Einfügen", for: .normal)
        knopfEinfügen.addTarget(self, action: #selector(Anzeige.EinfügenKnopfGedrückt), for: .touchUpInside)
        addSubview(knopfEinfügen)
        labelEntfernen.frame = CGRect(x: 10, y: 125, width: 380, height: 20)
        labelEntfernen.text = "Person aus Warteschlange entfernen"
        addSubview(labelEntfernen)
        labelNächster.frame = CGRect(x: 10, y: 160, width: 270, height: 20)
        labelNächster.text = "Zur Zeit kein Patient beim Arzt"
        addSubview(labelNächster)
        knopfEntfernen.frame = CGRect(x: 280, y: 155, width: 80, height: 30)
        knopfEntfernen.setTitle("Entfernen", for: .normal)
        knopfEntfernen.addTarget(self, action: #selector(Anzeige.EntfernenKnopfGedrückt), for: .touchUpInside)
        addSubview(knopfEntfernen)
    }
    
    required init?(coder: NSCoder)
    {
        fatalError("init(coder:) has not been implemented")
    }
    
    /**
    Aktionsfunktion für Einfügenknopf
    - parameters:
        - sender: der sendende Knopf
     */
    @objc func EinfügenKnopfGedrückt(sender: UIButton)
    {
        if let t = textfeldEinfügen.text
        {
            w.Einfügen(patientNeu: Patient(n: t))
            textfeldEinfügen.text = ""
        }
    }
    
    /**
    Aktionsfunktion für Einfügenknopf
    - parameters:
        - sender: der sendende Knopf
     */
    @objc func EntfernenKnopfGedrückt(sender: UIButton)
    {
        if let entfernter = w.Entfernen()
        {
            labelNächster.text = "Beim Arzt: " + entfernter.NameGeben()
        }
        else
        {
            labelNächster.text = "Zur Zeit kein Patient beim Arzt"
        }
    }
}

class MyViewController : UIViewController
{
    override func loadView()
    {
        let view = Anzeige(frame: CGRect(x: 0, y: 0, width: 400, height: 220))
        view.backgroundColor = .white
        self.view = view
    }
}

/**
Beschreibung der Klasse Patient

- author: Klaus Reinold
- version: 0.9
*/
class Patient
{
    /** Name des Patienten */
    private var name: String
    
    /**
    Konstruktor des Patienten
    - parameters:
        - n: Name des Patienten
    */
    init(n: String)
    {
        name = n;
    }

    /**
    Gibt Information über den Patienten aus.
    */
    func InformationAusgeben()
    {
        print(name)
    }
    
    /**
    Gibt den Namen des Patienten zurück
    - returns: Name des Patienten
    */
    func NameGeben() -> String
    {
        return name
    }
}

/**
Beschreibung der Warteschlange

- author: Klaus Reinold
- version: 0.9
*/
class Warteschlange
{
    /** Verwaltung der Patienten in der Warteschlange über ein Feld. */
    private var patientenfeld: [Patient?]
    
    /**
     Konstruktor der Warteschlange
     erzeugt ein leeres Feld
     */
    init()
    {
        patientenfeld = [Patient?]()
    }
    
    /**
     Fügt einen neuen Patienten am Ende der Warteschlange ein.
     
     Sucht von vorne den ersten freien Platz und fügt dort den Patienten ein.
     (in einem Feld können auch null-Werte auftreten)
     - parameters:
     - patientNeu: neuer Patient
     */
    func Einfügen(patientNeu: Patient)
    {
        for i in 0 ..< patientenfeld.count
        {
            if patientenfeld[i] == nil
            {
                patientenfeld[i] = patientNeu
                return
            }
        }
        patientenfeld.append(patientNeu)
    }
    
    /**
     Entfernt den ersten Patient aus der Warteschlange und gibt ihn aus.
     - returns: bisheriger erster Patient
     */
    func Entfernen() -> Patient?
    {
        //muss noch passend ergänzt werden!
        return nil
    }
}

PlaygroundPage.current.liveView = MyViewController()
